;print8x - USB Printer Driver and Interface
; (C) 2006 by Brandon Wilson. All rights reserved. 
;Portions in util.asm, usb8x.inc, equates.inc (C) Dan Englender.

 include "settings.inc" ;Specific settings for this application
 NOLIST
 include "ti83plus.inc"
 LIST
 include "equates.inc" ;Equates and macros to be used
 include "header.asm"
 GLOBALS ON
 include "usb8x.inc" ;usb8x equates

 SEGMENT MAIN

 EXTERN IPutS,IGetKey,DispHexA,DispHexHL,PrinterInit,GetPrinterCapabilities,GetPrinterStatus,USBErrorNoDrv,USBError
 EXTERN vputstring,StartPrintJob,SendPrintData,StopPrintJob,IPutS11,IsLanguageSupported,SoftKey,SendSoftReset

 Var USBDriverCall,32	; 32; RAM calls for U_CALL and callback routine
 Var USBDriverBuf,128	;128; Static buffer for USB driver's usage

;-----------------------------------------------------------------------------
Init:
       U_CALL_INIT DummyRet
       jr c,USBErrorNoDrv
       ld hl,USBDriverBuf
       U_CALL DriverInit
       jr c,USBError
       call PrinterInit
       jr c,USBError
refreshData:
       B_CALL ClrLCDFull
       B_CALL GrBufClr
       ld hl,0
       ld de,6*256+95
       B_CALL FillRect
       ld hl,3
       ld (penCol),hl
       ld hl,appBackUpScreen
       ld de,appBackUpScreen+64
       call GetPrinterCapabilities
       jr c,USBError
       ld hl,appBackUpScreen+64
       set textInverse,(iy+textFlags)
       call vputstring
       res textInverse,(iy+textFlags)
       call GetPrinterStatus
       jr c,USBError
       ld de,7*256+5
       ld (penCol),de
       ld hl,sLanguagesSupported
       call vputstring
       ld hl,appBackUpScreen
       ld de,14*256+10
       ld (penCol),de
langDispLoop:
       call IsLanguageSupported
       push af
       call vputstring
       pop af
       push hl
       ld hl,sNotSupported
       jr nc,langIsSupported
       jr $F
langIsSupported:
       ld hl,sSupported
$$:    call vputstring
       pop hl
       ld a,(penRow)
       add a,7
       ld (penRow),a
       ld a,10
       ld (penCol),a
       ld a,(hl)
       or a
       jr nz,langDispLoop
       call GetPrinterStatus
       jr c,USBError
       push af
       ld hl,48*256+5
       ld (penCol),hl
       ld hl,sStatus
       call vputstring
       pop af
       push af
       and 00000001b
       set textInverse,(iy+textFlags)
       jr z,$F
       res textInverse,(iy+textFlags)
$$:    ld hl,sErrorStatus
       call vputstring
       pop af
       push af
       and 00000010b
       res textInverse,(iy+textFlags)
       jr z,$F
       set textInverse,(iy+textFlags)
$$:    ld hl,sSelected
       call vputstring
       pop af
       and 00000100b
       set textInverse,(iy+textFlags)
       jr z,$F
       res textInverse,(iy+textFlags)
$$:    ld hl,sPaper
       call vputstring
       res textInverse,(iy+textFlags)
       ld hl,sMenuTable
       call SoftKey
mainKeyLoop:
       call IGetKey
       cp kMode
       jr z,AppExit
       cp kClear
       jr z,AppExit
       cp kQuit
       jr z,AppExit
       cp kEnter
       jr z,sendTestData
       cp kGraph
       jr z,AppExit
       cp kYequ
       jr z,refreshData
       cp kWindow
       jr nz,mainKeyLoop
       call SendSoftReset
       jr refreshData
sendTestData:
       call StartPrintJob
       ld hl,sData
       ld bc,sDataEnd-sData
       call SendPrintData
       call StopPrintJob
       jr mainKeyLoop
AppExit:
       B_CALL ClrLCDFull
       B_CALL HomeUp
       U_CALL HostKill
       U_CALL DriverKill
       B_JUMP JForceCmdNoChar
DummyRet:
       ret

sLanguagesSupported:
       DB "Languages supported: ",0
sNotSupported:
       DB " - NOT IMPLEMENTED",0
sSupported:
       DB " - Implemented",0
sMenuTable:
       DB "Updt",0
       DB "Rset",0
       DB 0
       DB 0
       DB "Quit",0
sStatus:
       DB "Status: ",0
sErrorStatus:
       DB 6,"ERR",6,0
sSelected:
       DB 6,"SEL",6,0
sPaper:
       DB 6,"PAPER",6,0
sData:
       DB 1Bh,"E","This is a test of the emergency broadcast system.",1Bh,"E"
sDataEnd:
